@@ -1,6 +1,6 @@
 # --
 # Kernel/System/DB/oracle.pm - oracle database backend
-# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -33,9 +33,13 @@ sub LoadPreferences {
     $Self->{'DB::QuoteSemicolon'}       = '';
     $Self->{'DB::QuoteUnderscoreStart'} = '\\';
     $Self->{'DB::QuoteUnderscoreEnd'}   = '';
-    $Self->{'DB::CaseInsensitive'}      = 0;
+    $Self->{'DB::CaseSensitive'}        = 1;
     $Self->{'DB::LikeEscapeString'}     = q{ESCAPE '\\'};
 
+    # how to determine server version
+    $Self->{'DB::Version'}
+        = "SELECT CONCAT('Oracle ', version) FROM product_component_version WHERE product LIKE 'Oracle Database%'";
+
     # dbi attributes
     $Self->{'DB::Attribute'} = {
         LongTruncOk => 1,
@@ -49,12 +53,13 @@ sub LoadPreferences {
     $Self->{'DB::Encode'} = 0;
 
     # shell setting
-    $Self->{'DB::Comment'}      = '-- ';
-    $Self->{'DB::ShellCommit'}  = ';';
-    $Self->{'DB::ShellConnect'} = "SET DEFINE OFF;\nSET SQLBLANKLINES ON"; # must be on separate lines!
+    $Self->{'DB::Comment'}     = '-- ';
+    $Self->{'DB::ShellCommit'} = ';';
+    $Self->{'DB::ShellConnect'}
+        = "SET DEFINE OFF;\nSET SQLBLANKLINES ON";    # must be on separate lines!
 
     # init sql setting on db connect
-    #$Self->{'DB::Connect'} = '';
+    $Self->{'DB::Connect'} = "ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'";
 
     return 1;
 }
@@ -239,8 +244,16 @@ sub TableCreate {
             if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
                 $Shell = "/\n--";
             }
-            push( @Return2, "DROP SEQUENCE $Sequence" );
-            push( @Return2, "CREATE SEQUENCE $Sequence" );
+            push(
+                @Return2,
+                "CREATE SEQUENCE $Sequence\n"
+                    . "INCREMENT BY 1\n"
+                    . "START WITH 1\n"
+                    . "NOMAXVALUE\n"
+                    . "NOCYCLE\n"
+                    . "CACHE 20\n"
+                    . "ORDER",
+            );
             push(
                 @Return2,
                 "CREATE OR REPLACE TRIGGER $Sequence"
@@ -795,9 +808,6 @@ sub Insert {
             }
         }
         else {
-            if ( $Self->{ConfigObject}->Get('Database::ShellOutput') ) {
-                $Tmp =~ s/\n/\r/g;
-            }
             $Value .= $Tmp;
         }
     }
