@@ -1,9 +1,10 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/DynamicField/Driver/Base.pm - Dynamic field backend functions
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::DynamicField::Driver::Base;
@@ -13,17 +14,16 @@ use warnings;
 
 use Kernel::System::VariableCheck qw(:all);
 
-our @ObjectDependencies = (
-    'Kernel::System::DynamicFieldValue',
-    'Kernel::System::Log',
-);
-
 =head1 NAME
 
 Kernel::System::DynamicField::Driver::Base - common fields backend functions
 
+=head1 SYNOPSIS
+
 =head1 PUBLIC INTERFACE
 
+=over 4
+
 =cut
 
 sub ValueIsDifferent {
@@ -31,19 +31,16 @@ sub ValueIsDifferent {
 
     # special cases where the values are different but they should be reported as equals
     return if !defined $Param{Value1} && ( defined $Param{Value2} && $Param{Value2} eq '' );
-    return if !defined $Param{Value2} && ( defined $Param{Value1} && $Param{Value1} eq '' );
+    return if !defined $Param{Value2} && $Param{Value1} eq '';
 
     # compare the results
-    return DataIsDifferent(
-        Data1 => \$Param{Value1},
-        Data2 => \$Param{Value2}
-    );
+    return DataIsDifferent( Data1 => \$Param{Value1}, Data2 => \$Param{Value2} );
 }
 
 sub ValueDelete {
     my ( $Self, %Param ) = @_;
 
-    my $Success = $Kernel::OM->Get('Kernel::System::DynamicFieldValue')->ValueDelete(
+    my $Success = $Self->{DynamicFieldValueObject}->ValueDelete(
         FieldID  => $Param{DynamicFieldConfig}->{ID},
         ObjectID => $Param{ObjectID},
         UserID   => $Param{UserID},
@@ -55,7 +52,7 @@ sub ValueDelete {
 sub AllValuesDelete {
     my ( $Self, %Param ) = @_;
 
-    my $Success = $Kernel::OM->Get('Kernel::System::DynamicFieldValue')->AllValuesDelete(
+    my $Success = $Self->{DynamicFieldValueObject}->AllValuesDelete(
         FieldID => $Param{DynamicFieldConfig}->{ID},
         UserID  => $Param{UserID},
     );
@@ -71,6 +68,9 @@ sub HasBehavior {
 
     # return success if the dynamic field has the expected behavior
     return IsPositiveInteger( $Self->{Behaviors}->{ $Param{Behavior} } );
+
+    # otherwise return fail
+    return;
 }
 
 sub SearchFieldPreferences {
@@ -86,7 +86,7 @@ sub SearchFieldPreferences {
     return \@Preferences;
 }
 
-=head2 EditLabelRender()
+=item EditLabelRender()
 
 creates the label HTML to be used in edit masks.
 
@@ -105,17 +105,14 @@ sub EditLabelRender {
     # check needed stuff
     for my $Needed (qw(DynamicFieldConfig FieldName)) {
         if ( !$Param{$Needed} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $Needed!"
-            );
+            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $Needed!" );
             return;
         }
     }
 
     # check DynamicFieldConfig (general)
     if ( !IsHashRefWithData( $Param{DynamicFieldConfig} ) ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $Self->{LogObject}->Log(
             Priority => 'error',
             Message  => "The field configuration is invalid",
         );
@@ -125,7 +122,7 @@ sub EditLabelRender {
     # check DynamicFieldConfig (internally)
     for my $Needed (qw(Label)) {
         if ( !$Param{DynamicFieldConfig}->{$Needed} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
+            $Self->{LogObject}->Log(
                 Priority => 'error',
                 Message  => "Need $Needed in DynamicFieldConfig!"
             );
@@ -156,17 +153,17 @@ EOF
     }
 
     # text
-    $HTMLString .= $Param{LayoutObject}->Ascii2Html(
-        Text => $Param{LayoutObject}->{LanguageObject}->Translate("$LabelText")
-    );
     if ( $Param{AdditionalText} ) {
-        $HTMLString .= " (";
-        $HTMLString .= $Param{LayoutObject}->Ascii2Html(
-            Text => $Param{LayoutObject}->{LanguageObject}->Translate("$Param{AdditionalText}")
-        );
-        $HTMLString .= ")";
+        $HTMLString .= <<"EOF";
+    \$Text{"$LabelText"} (\$Text{"$Param{AdditionalText}"}):
+EOF
+
+    }
+    else {
+        $HTMLString .= <<"EOF";
+    \$Text{"$LabelText"}:
+EOF
     }
-    $HTMLString .= ":\n";
 
     # closing tag
     $HTMLString .= <<"EOF";
@@ -176,79 +173,16 @@ EOF
     return $HTMLString;
 }
 
-=head2 ValueSearch()
-
-Searches/fetches dynamic field value.
-
-    my $Value = $BackendObject->ValueSearch(
-        DynamicFieldConfig => $DynamicFieldConfig,      # complete config of the DynamicField
-        Search             => 'test',
-    );
-
-    Returns [
-        {
-            ID            => 437,
-            FieldID       => 23,
-            ObjectID      => 133,
-            ValueText     => 'some text',
-            ValueDateTime => '1977-12-12 12:00:00',
-            ValueInt      => 123,
-        },
-    ];
-
-=cut
-
-sub ValueSearch {
-    my ( $Self, %Param ) = @_;
-
-    # check mandatory parameters
-    if ( !IsHashRefWithData( $Param{DynamicFieldConfig} ) ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Need DynamicFieldConfig!"
-        );
-        return;
-    }
-
-    my $SearchTerm = $Param{Search};
-    my $Operator   = 'Equals';
-    if ( $Self->HasBehavior( Behavior => 'IsLikeOperatorCapable' ) ) {
-        $SearchTerm = '%' . $Param{Search} . '%';
-        $Operator   = 'Like';
-    }
-
-    my $SearchSQL = $Self->SearchSQLGet(
-        DynamicFieldConfig => $Param{DynamicFieldConfig},
-        TableAlias         => 'dynamic_field_value',
-        SearchTerm         => $SearchTerm,
-        Operator           => $Operator,
-    );
-
-    if ( !defined $SearchSQL || !length $SearchSQL ) {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Error generating search SQL!"
-        );
-        return;
-    }
-
-    my $Values = $Kernel::OM->Get('Kernel::System::DynamicFieldValue')->ValueSearch(
-        FieldID   => $Param{DynamicFieldConfig}->{ID},
-        Search    => $Param{Search},
-        SearchSQL => $SearchSQL,
-    );
-
-    return $Values;
-}
-
 1;
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
