@@ -1,6 +1,6 @@
 # --
 # Kernel/System/Package.pm - lib package manager
-# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -21,6 +21,10 @@ use Kernel::System::Loader;
 use Kernel::System::SysConfig;
 use Kernel::System::WebUserAgent;
 use Kernel::System::XML;
+use Kernel::System::EventHandler;
+use Kernel::System::VariableCheck qw(:all);
+
+use base qw(Kernel::System::EventHandler);
 
 =head1 NAME
 
@@ -137,7 +141,7 @@ sub new {
     $Self->{PackageMapFileList} = { File => 'ARRAY', };
 
     $Self->{PackageVerifyURL}
-        = 'https://pav.otrs.com/otrs/public.pl?Action=PublicPackageVerification;';
+        = 'https://pav.otrs.com/otrs/public.pl';
 
     $Self->{Home} = $Self->{ConfigObject}->Get('Home');
 
@@ -147,13 +151,22 @@ sub new {
             . "Try: \$OTRS_HOME/bin/otrs.SetPermissions.pl !!!\n";
     }
 
+    # init of event handler
+    $Self->EventHandlerInit(
+        Config     => 'Package::EventModule',
+        BaseObject => 'PackageObject',
+        Objects    => {
+            %{$Self},
+        },
+    );
+
     return $Self;
 }
 
 =item RepositoryList()
 
 returns a list of repository packages
-using Result => 'short' will only return name, version, install_status and md5sums
+using Result => 'short' will only return name, version, install_status md5sum and vendor
 instead of the structure
 
     my @List = $PackageObject->RepositoryList();
@@ -181,7 +194,7 @@ sub RepositoryList {
 
     # get repository list
     $Self->{DBObject}->Prepare(
-        SQL => 'SELECT name, version, install_status, content '
+        SQL => 'SELECT name, version, install_status, content, vendor '
             . 'FROM package_repository ORDER BY name, create_time',
     );
 
@@ -192,16 +205,24 @@ sub RepositoryList {
             Name    => $Row[0],
             Version => $Row[1],
             Status  => $Row[2],
+            Vendor  => $Row[4],
         );
 
+        # correct any 'dos-style' line endings - http://bugs.otrs.org/show_bug.cgi?id=9838
+        $Row[3] =~ s{\r\n}{\n}xmsg;
+        $Package{MD5sum} = $Self->{MainObject}->MD5sum( String => \$Row[3] );
+
         # get package attributes
         if ( $Row[3] && $Result eq 'Short' ) {
-            $Package{MD5sum} = $Self->{MainObject}->MD5sum( String => \$Row[3] );
+
             push @Data, {%Package};
+
         }
         elsif ( $Row[3] ) {
+
             my %Structure = $Self->PackageParse( String => \$Row[3] );
             push @Data, { %Package, %Structure };
+
         }
     }
 
@@ -305,6 +326,11 @@ sub RepositoryAdd {
 
     # get package attributes
     my %Structure = $Self->PackageParse(%Param);
+
+    if ( !IsHashRefWithData( \%Structure ) ) {
+        $Self->{LogObject}->Log( Priority => 'error', Message => 'Invalid Package!' );
+        return;
+    }
     if ( !$Structure{Name} ) {
         $Self->{LogObject}->Log( Priority => 'error', Message => 'Need Name!' );
         return;
@@ -526,6 +552,17 @@ sub PackageInstall {
     $Self->{CacheObject}->CleanUp();
     $Self->{LoaderObject}->CacheDelete();
 
+    # trigger event
+    $Self->EventHandler(
+        Event => 'PackageInstall',
+        Data  => {
+            Name    => $Structure{Name}->{Content},
+            Vendor  => $Structure{Vendor}->{Content},
+            Version => $Structure{Version}->{Content},
+        },
+        UserID => 1,
+    );
+
     return 1;
 }
 
@@ -593,6 +630,17 @@ sub PackageReinstall {
     $Self->{CacheObject}->CleanUp();
     $Self->{LoaderObject}->CacheDelete();
 
+    # trigger event
+    $Self->EventHandler(
+        Event => 'PackageReinstall',
+        Data  => {
+            Name    => $Structure{Name}->{Content},
+            Vendor  => $Structure{Vendor}->{Content},
+            Version => $Structure{Version}->{Content},
+        },
+        UserID => 1,
+    );
+
     return 1;
 }
 
@@ -886,6 +934,17 @@ sub PackageUpgrade {
     $Self->{CacheObject}->CleanUp();
     $Self->{LoaderObject}->CacheDelete();
 
+    # trigger event
+    $Self->EventHandler(
+        Event => 'PackageUpgrade',
+        Data  => {
+            Name    => $Structure{Name}->{Content},
+            Vendor  => $Structure{Vendor}->{Content},
+            Version => $Structure{Version}->{Content},
+        },
+        UserID => 1,
+    );
+
     return 1;
 }
 
@@ -965,6 +1024,17 @@ sub PackageUninstall {
     $Self->{CacheObject}->CleanUp();
     $Self->{LoaderObject}->CacheDelete();
 
+    # trigger event
+    $Self->EventHandler(
+        Event => 'PackageUninstall',
+        Data  => {
+            Name    => $Structure{Name}->{Content},
+            Vendor  => $Structure{Vendor}->{Content},
+            Version => $Structure{Version}->{Content},
+        },
+        UserID => 1,
+    );
+
     return 1;
 }
 
@@ -1400,13 +1470,18 @@ sub PackageVerify {
 
     # define package verification info
     my $PackageVerifyInfo = {
-        Description => "<br>If you continue to install this package, the following issues may occur!<br><br>&nbsp;-Security problems<br>&nbsp;-Stability problems<br>&nbsp;-Performance problems<br><br>Please note that issues that are caused by working with this package are not covered by OTRS service contracts!<br><br>",
-        Title       => 'Package not verified by the OTRS Group! It is recommended not to use this package.',
+        Description =>
+            "<br>If you continue to install this package, the following issues may occur!<br><br>&nbsp;-Security problems<br>&nbsp;-Stability problems<br>&nbsp;-Performance problems<br><br>Please note that issues that are caused by working with this package are not covered by OTRS service contracts!<br><br>",
+        Title =>
+            'Package not verified by the OTRS Group! It is recommended not to use this package.',
     };
 
     # investigate name
     my $Name = $Param{Structure}->{Name}->{Content} || $Param{Name};
 
+    # correct any 'dos-style' line endings - http://bugs.otrs.org/show_bug.cgi?id=9838
+    $Param{Package} =~ s{\r\n}{\n}xmsg;
+
     # create MD5 sum
     my $Sum = $Self->{MainObject}->MD5sum( String => $Param{Package} );
 
@@ -1433,9 +1508,13 @@ sub PackageVerify {
 
     # verify package at web server
     my %Response = $WebUserAgentObject->Request(
-        URL => $Self->{PackageVerifyURL} . 'Package=' . $Name . '::' . $Sum,
+        URL  => $Self->{PackageVerifyURL},
+        Type => 'POST',
+        Data => {
+            Action  => 'PublicPackageVerification',
+            Package => $Name . '::' . $Sum,
+            }
     );
-
     return 'verified' if !$Response{Status};
     return 'verified' if $Response{Status} ne '200 OK';
     return 'verified' if !$Response{Content};
@@ -1465,7 +1544,7 @@ sub PackageVerify {
         Type  => 'PackageVerification',
         Key   => $Sum,
         Value => $PackageVerify,
-        TTL   => 30 * 24 * 60 * 60,  # 30 days
+        TTL   => 30 * 24 * 60 * 60,       # 30 days
     );
 
     return $PackageVerify;
@@ -1536,14 +1615,13 @@ sub PackageVerifyAll {
         }
         else {
             $Result{ $Package->{Name} } = 'verified';
-            push @PackagesToVerify, 'Package=' . $Package->{Name} . '::' . $Package->{MD5sum};
+            push @PackagesToVerify, 'Package';
+            push @PackagesToVerify, $Package->{Name} . '::' . $Package->{MD5sum};
         }
     }
 
     return %Result if !@PackagesToVerify;
 
-    my $PackagesString = join ';', @PackagesToVerify;
-
     # create new web user agent object -> note proxy is different from Package::Proxy
     my $WebUserAgentObject = Kernel::System::WebUserAgent->new(
         DBObject     => $Self->{DBObject},
@@ -1557,7 +1635,12 @@ sub PackageVerifyAll {
 
     # verify package at web server
     my %Response = $WebUserAgentObject->Request(
-        URL => $Self->{PackageVerifyURL} . $PackagesString,
+        URL  => $Self->{PackageVerifyURL},
+        Type => 'POST',
+        Data => [
+            Action => 'PublicPackageVerification',
+            @PackagesToVerify
+            ]
     );
 
     return %Result if !$Response{Status};
@@ -1587,13 +1670,13 @@ sub PackageVerifyAll {
 
         # process result
         $Result{$Package} = $PackageVerify;
-        
+
         # set cache
         $Self->{CacheObject}->Set(
             Type  => 'PackageVerification',
             Key   => $PackageList{$Package},
             Value => $PackageVerify,
-            TTL   => 30 * 24 * 60 * 60,  # 30 days
+            TTL   => 30 * 24 * 60 * 60,        # 30 days
         );
     }
 
@@ -1629,7 +1712,7 @@ build an opm package
                 Lang    => 'de',
                 Content => 'german description',
             },
-d        ],
+        ],
         Filelist = [
             {
                 Location   => 'Kernel/System/Lala.pm'
@@ -1747,12 +1830,6 @@ sub PackageBuild {
                 $XML .= $Self->_Encode( $OldParam{Content} ) . "</$TagSub>\n";
             }
         }
-        else {
-
-#            $XML .= "  <$Tag></$Tag>\n";
-#            $Self->{LogObject}->Log(Priority => 'error', Message => "Invalid Ref data in tag $Tag!");
-#            return;
-        }
     }
 
     # don't use Build* in index mode
@@ -1927,7 +2004,7 @@ sub PackageParse {
 
     $Self->{EncodeObject}->EncodeOutput( \$CookedString );
 
-    # create cecksum
+    # create checksum
     my $Checksum = $Self->{MainObject}->MD5sum(
         String => \$CookedString,
     );
@@ -1941,7 +2018,17 @@ sub PackageParse {
         return %{$Cache} if $Cache;
     }
 
-    my @XMLARRAY = $Self->{XMLObject}->XMLParse(%Param);
+    my @XMLARRAY = eval {
+        $Self->{XMLObject}->XMLParse(%Param);
+    };
+
+    if ( !IsArrayRefWithData( \@XMLARRAY ) ) {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message  => "Invalid XMLParse in PackageParse()!",
+        );
+        return;
+    }
 
     # cleanup global vars
     undef $Self->{Package};
@@ -2060,7 +2147,16 @@ sub PackageParse {
         }
     }
 
-    # return package structur
+    # check if a structure is present
+    if ( !IsHashRefWithData( $Self->{Package} ) ) {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message  => "Invalid package structure in PackageParse()!",
+        );
+        return;
+    }
+
+    # return package structure
     my %Return = %{ $Self->{Package} };
     undef $Self->{Package};
 
@@ -2191,7 +2287,7 @@ sub PackageInstallDefaultFiles {
 
         next LOCATION if !$ContentSCALARRef;
 
-        # install package (use eval to be save)
+        # install package (use eval to be safe)
         eval {
             $Self->PackageInstall( String => ${$ContentSCALARRef} );
         };
@@ -2287,7 +2383,7 @@ sub _Code {
         return;
     }
 
-    # code exec
+    # execute code
     CODE:
     for my $Code ( @{ $Param{Code} } ) {
 
@@ -2464,7 +2560,7 @@ sub _CheckVersion {
     # if it is not an external package, and the versions are different
     # we want to add a 0 at the end of the shorter version number
     # (1.2.3 will be modified to 1.2.3.0)
-    # This is important to compare with a test releaseversion number
+    # This is important to compare with a test-release version number
     if ( !$Param{ExternalPackage} && $Parts{VersionNewNum} ne $Parts{VersionInstalledNum} ) {
 
         TYPE:
@@ -2675,7 +2771,7 @@ sub _PackageFileCheck {
         return;
     }
 
-    # check if one of this files is already intalled by an other package
+    # check if one of the files is already installed by another package
     PACKAGE:
     for my $Package ( $Self->RepositoryList() ) {
 
@@ -2745,7 +2841,7 @@ sub _FileInstall {
         }
         else {
 
-            # check if we reinstall the same file, create a .save it not the same one
+            # check if we reinstall the same file, create a .save if it is not the same
             my $Save = 0;
             if ( $Param{Reinstall} && !-e "$RealFile.save" ) {
 
@@ -2756,7 +2852,7 @@ sub _FileInstall {
                 );
                 if ( $Content && ${$Content} ne $Param{File}->{Content} ) {
 
-                    # check if it's framework file, create .save file
+                    # check if it's a framework file, create .save file
                     my %File = $Self->_ReadDistArchive( Home => $Home );
                     if ( $File{ $Param{File}->{Location} } ) {
                         $Save = 1;
@@ -2863,7 +2959,7 @@ sub _FileRemove {
         }
     }
 
-    # check if it's framework file and if $RealFile.(backup|save) exists
+    # check if it's a framework file and if $RealFile.(backup|save) exists
     # then do not remove it!
     my %File = $Self->_ReadDistArchive( Home => $Home );
     if ( $File{ $Param{File}->{Location} } && ( !-e "$RealFile.backup" && !-e "$RealFile.save" ) ) {
@@ -2997,6 +3093,143 @@ sub _Encode {
     return $Text;
 }
 
+=item _PackageUninstallMerged()
+
+ONLY CALL THIS METHOD FROM A DATABASE UPGRADING SCRIPT DURING FRAMEWORK UPDATES
+OR FROM A CODEUPGRADE SECTION IN AN SOPM FILE OF A PACKAGE THAT INCLUDES A MERGED FEATURE ADDON.
+
+Uninstall an already framework (or module) merged package.
+
+Package files that are not in the framework ARCHIVE file will be deleted, DatabaseUninstall() and
+CodeUninstall are not called.
+
+    $Success = $PackageObject->_PackageUninstallMerged(
+        Name        => 'some package name',
+        Home        => 'OTRS Home path',      # Optional
+        DeleteSaved => 1,                     # or 0, 1 Default, Optional: if set to 1 it also
+                                              # delete .save files
+    );
+
+=cut
+
+sub _PackageUninstallMerged {
+    my ( $Self, %Param ) = @_;
+
+    # check needed stuff
+    if ( !$Param{Name} ) {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message  => 'Need Name (Name of the package)!'
+        );
+        return;
+    }
+
+    my $Home = $Param{Home} || $Self->{Home};
+
+    # check Home
+    if ( !-e $Home ) {
+        $Self->{LogObject}->Log(
+            Priority => 'error',
+            Message  => "No such home directory: $Home!",
+        );
+        return;
+    }
+
+    if ( !defined $Param{DeleteSaved} ) {
+        $Param{DeleteSaved} = 1;
+    }
+
+    # check if the package is installed, otherwise return success (nothing to do)
+    my $PackageInstalled = $Self->PackageIsInstalled(
+        Name => $Param{Name},
+    );
+    return 1 if !$PackageInstalled;
+
+    # get the package details
+    my @PackageList       = $Self->RepositoryList();
+    my %PackageListLookup = map { $_->{Name}->{Content} => $_ } @PackageList;
+    my %PackageDetails    = %{ $PackageListLookup{ $Param{Name} } };
+
+    # get the list of framework files
+    my %FrameworkFiles = $Self->_ReadDistArchive( Home => $Home );
+
+    # can not continue if there are no framework files
+    return if !%FrameworkFiles;
+
+    # remove unneeded files (if exists)
+    if ( IsArrayRefWithData( $PackageDetails{Filelist} ) ) {
+
+        FILE:
+        for my $FileHash ( @{ $PackageDetails{Filelist} } ) {
+
+            my $File = $FileHash->{Location};
+
+            # get real file name in fs
+            my $RealFile = $Home . '/' . $File;
+            $RealFile =~ s/\/\//\//g;
+
+            # check if file exists
+            if ( -e $RealFile ) {
+
+                # check framework files (use $File instead of $RealFile)
+                if ( $FrameworkFiles{$File} ) {
+
+                    if ( $Param{DeleteSaved} ) {
+
+                        # check if file was overridden by the package
+                        my $SavedFile = $RealFile . '.save';
+                        if ( -e $SavedFile ) {
+
+                            # remove old file
+                            if ( !$Self->{MainObject}->FileDelete( Location => $SavedFile ) ) {
+                                $Self->{LogObject}->Log(
+                                    Priority => 'error',
+                                    Message  => "Can't remove file $SavedFile: $!!",
+                                );
+                                return;
+                            }
+                            print STDERR "Notice: Removed old backup file: $SavedFile\n";
+                        }
+                    }
+
+                    # skip framework file
+                    print STDERR "Notice: Skiped framework file: $RealFile\n";
+                    next FILE;
+                }
+
+                # remove old file
+                if ( !$Self->{MainObject}->FileDelete( Location => $RealFile ) ) {
+                    $Self->{LogObject}->Log(
+                        Priority => 'error',
+                        Message  => "Can't remove file $RealFile: $!!",
+                    );
+                    return;
+                }
+                print STDERR "Notice: Removed file: $RealFile\n";
+            }
+        }
+    }
+
+    # delete package from the database
+    my $PackageRemove = $Self->RepositoryRemove(
+        Name => $Param{Name},
+    );
+
+    $Self->{CacheObject}->CleanUp();
+    $Self->{LoaderObject}->CacheDelete();
+
+    return $PackageRemove;
+}
+
+sub DESTROY {
+    my $Self = shift;
+
+    # execute all transaction events
+    $Self->EventHandlerTransaction();
+
+    return 1;
+}
+
 1;
 
 =end Internal:
