@@ -1,9 +1,10 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/SupportDataCollector/Plugin/Webserver/Apache/Performance.pm - system data collector plugin
+# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::SupportDataCollector::Plugin::Webserver::Apache::Performance;
@@ -11,14 +12,10 @@ package Kernel::System::SupportDataCollector::Plugin::Webserver::Apache::Perform
 use strict;
 use warnings;
 
-use parent qw(Kernel::System::SupportDataCollector::PluginBase);
-
-use Kernel::Language qw(Translatable);
-
-our @ObjectDependencies = ();
+use base qw(Kernel::System::SupportDataCollector::PluginBase);
 
 sub GetDisplayPath {
-    return Translatable('Webserver');
+    return 'Webserver';
 }
 
 sub Run {
@@ -26,90 +23,71 @@ sub Run {
 
     my %Environment = %ENV;
 
-    # No web request or no apache webserver, skip this check.
-    if ( !$ENV{GATEWAY_INTERFACE} || !$ENV{SERVER_SOFTWARE} || $ENV{SERVER_SOFTWARE} !~ m{apache}i ) {
+    # No apache webserver, skip this check
+    if ( !$ENV{SERVER_SOFTWARE} || $ENV{SERVER_SOFTWARE} !~ m{apache}i ) {
         return $Self->GetResults();
     }
 
+    my @Result;
+
     # Check for CGI accelerator
     if ( $ENV{MOD_PERL} ) {
         $Self->AddResultOk(
             Identifier => "CGIAcceleratorUsed",
-            Label      => Translatable('CGI Accelerator Usage'),
+            Label      => 'CGI Accelerator Usage',
             Value      => $ENV{MOD_PERL},
         );
     }
-    elsif ( $INC{'CGI/Fast.pm'} || $ENV{FCGI_ROLE} || $ENV{FCGI_SOCKET_PATH} ) {
+    elsif ( $ENV{SERVER_SOFTWARE} =~ /fastcgi/i ) {
         $Self->AddResultOk(
             Identifier => "CGIAcceleratorUsed",
-            Label      => Translatable('CGI Accelerator Usage'),
+            Label      => 'CGI Accelerator Usage',
             Value      => 'fastcgi',
         );
     }
     else {
         $Self->AddResultWarning(
             Identifier => "CGIAcceleratorUsed",
-            Label      => Translatable('CGI Accelerator Usage'),
+            Label      => 'CGI Accelerator Usage',
             Value      => '',
-            Message    => Translatable('You should use FastCGI or mod_perl to increase your performance.'),
+            Message    => 'You should use FastCGI or mod_perl to increase your performance.',
         );
     }
 
     if ( $ENV{MOD_PERL} ) {
-        my $ModDeflateLoaded =
-            Apache2::Module::loaded('mod_deflate.c') || Apache2::Module::loaded('mod_deflate.so');
+        my $ModDeflateLoaded = Apache2::Module::loaded('mod_deflate.c');
 
         if ($ModDeflateLoaded) {
             $Self->AddResultOk(
                 Identifier => "ModDeflateLoaded",
-                Label      => Translatable('mod_deflate Usage'),
+                Label      => 'mod_deflate.c Usage',
                 Value      => 'active',
             );
         }
         else {
             $Self->AddResultWarning(
                 Identifier => "ModDeflateLoaded",
-                Label      => Translatable('mod_deflate Usage'),
+                Label      => 'mod_deflate Usage',
                 Value      => 'not active',
-                Message    => Translatable('Please install mod_deflate to improve GUI speed.'),
+                Message    => 'Please install mod_deflate to improve GUI speed.',
             );
         }
 
-        my $ModFilterLoaded =
-            Apache2::Module::loaded('mod_filter.c') || Apache2::Module::loaded('mod_filter.so');
-
-        if ($ModFilterLoaded) {
-            $Self->AddResultOk(
-                Identifier => "ModFilterLoaded",
-                Label      => Translatable('mod_filter Usage'),
-                Value      => 'active',
-            );
-        }
-        else {
-            $Self->AddResultWarning(
-                Identifier => "ModFilterLoaded",
-                Label      => Translatable('mod_filter Usage'),
-                Value      => 'not active',
-                Message    => Translatable('Please install mod_filter if mod_deflate is used.'),
-            );
-        }
-
-        my $ModHeadersLoaded =
-            Apache2::Module::loaded('mod_headers.c') || Apache2::Module::loaded('mod_headers.so');
+        my $ModHeadersLoaded = Apache2::Module::loaded('mod_headers.c');
 
         if ($ModHeadersLoaded) {
             $Self->AddResultOk(
                 Identifier => "ModHeadersLoaded",
-                Label      => Translatable('mod_headers Usage'),
+                Label      => 'mod_headers.c Usage',
                 Value      => 'active',
             );
         }
         else {
             $Self->AddResultWarning(
                 Identifier => "ModHeadersLoaded",
-                Label      => Translatable('mod_headers Usage'),
+                Label      => 'mod_headers Usage',
                 Value      => 'not active',
-                Message    => Translatable('Please install mod_headers to improve GUI speed.'),
+                Message    => 'Please install mod_headers to improve GUI speed.',
             );
         }
 
@@ -126,19 +104,17 @@ sub Run {
         if ($ApacheReloadUsed) {
             $Self->AddResultOk(
                 Identifier => "ApacheReloadUsed",
-                Label      => Translatable('Apache::Reload Usage'),
+                Label      => 'Apache::Reload Usage',
                 Value      => 'active',
             );
         }
         else {
             $Self->AddResultWarning(
                 Identifier => "ApacheReloadUsed",
-                Label      => Translatable('Apache::Reload Usage'),
+                Label      => 'Apache::Reload Usage',
                 Value      => 'not active',
                 Message =>
-                    Translatable(
-                    'Apache::Reload or Apache2::Reload should be used as PerlModule and PerlInitHandler to prevent web server restarts when installing and upgrading modules.'
-                    ),
+                    'Apache::Reload or Apache2::Reload should be used as PerlModule and PerlInitHandler to prevent web server restarts when installing and upgrading modules.',
             );
         }
 
@@ -154,25 +130,35 @@ sub Run {
         if ($ApacheDBIUsed) {
             $Self->AddResultOk(
                 Identifier => "ApacheDBIUsed",
-                Label      => Translatable('Apache2::DBI Usage'),
+                Label      => 'Apache2::DBI Usage',
                 Value      => 'active',
             );
         }
         else {
             $Self->AddResultWarning(
                 Identifier => "ApacheDBIUsed",
-                Label      => Translatable('Apache2::DBI Usage'),
+                Label      => 'Apache2::DBI Usage',
                 Value      => 'not active',
                 Message =>
-                    Translatable(
-                    'Apache2::DBI should be used to get a better performance  with pre-established database connections.'
-                    ),
+                    'Apache2::DBI should be used to get a better performance  with pre-established database connections.',
             );
         }
 
     }
 
-    return $Self->GetResults();
+    return $Self->GetResults()
 }
 
+=back
+
+=head1 TERMS AND CONDITIONS
+
+This software is part of the OTRS project (L<http://otrs.org/>).
+
+This software comes with ABSOLUTELY NO WARRANTY. For details, see
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
+
+=cut
+
 1;
